// Functionality to create walls and fortresses
// Wall object
function wallElement(type, entity, angle, width, indent, bending)
{
	// NOTE: Not all wall elements have a symetry. So there's an direction 'outside' (towards top/positive Z[2 dimensional Y] by default)
	// In this sense 'right'/'left' means right/left of you when you stand upon the wall and look 'outside'
	// The wall is build towards 'right' so the next wall element will be placed right of the previous one
	// In this sense the wall's direction is right meaning the 'bending' of a corner is used for the element following the corner
	// With 'inside' and 'outside' defined as above, corners bend 'in'/'out' meaning placemant angle is increased/decreased (circlewise like building placement)
	this.type = type; // Descriptive string for easy to add wall styles. Not really needed (wallTool.wallTypes['type string'] is used). May be helpfull for custom wall elements though.
	// Wall type documentation:
	// wall: A blocking straight wall element that mainly lengthens the wall, self-explanatory
	// tower: A blocking straight wall element with damage potential that slightly lengthens the wall, exsample: wall tower, palisade tower(No attack?)
	// wallFort: A blocking straight wall element with massive damage potential that lengthens the wall, exsample: fortress, palisade fort
		// NOTE: Not really needed but for palisades, I don't find it tragic that palisades are much less powerfull
	// outpost: A zero-length wall element without bending far indented so it stands outside the wall, exsample: outpost, defense tower, watchtower
	// house: A zero-length wall element without bending far indented so it stands inside the wall, exsample: house, hut, longhouse
	// gate: A blocking straight wall element with passability determined by owner, example: gate
	// entry: A non-blocking straight wall element represented by a single indented template, example: defense tower, wall tower, outpost, watchtower, nothing
	// wideEntry: A non-blocking straight wall element represented by a single indented template, example: fortress? a unit?
		// NOTE: Not really needed...
	// endRight: A straight wall element that (only) visually ends the wall to the right (e.g. left of entrys), example: wall tower, palisade ending
	// endLeft: A straight wall element that (only) visually ends the wall to the left (e.g. right of entrys), example: wall tower, palisade ending
	// cornerIn: A wall element bending the wall PI/2 (90°) 'inside' (see above), example: wall tower, palisade curve
	// cornerOut: A wall element bending the wall PI/2 (90°) 'outside' (see above), example: wall tower, palisade curve (placed the other way arround)
	// barracks: Adding in progress...
	this.entity = undefined; // Optional. Template (string) that will represent the wall element, example: "other/palisades_rocks_straight"
	if (entity !== undefined)
		this.entity = entity;
	this.angle = 0*PI; // Optional. Placement angle so that 'outside' is 'top' (directs towards positive z, like buildings)
	if (angle !== undefined)
		this.angle = angle;
	this.width = 0; // Optional. The width it lengthens the wall, width because it's the needed space in a right angle to 'outside'
	if (width !== undefined)
		this.width = width;
	this.indent = 0; // Optional. The indentation means its drawn inside (positive values) or pressed outwards (negative values)
	if (indent !== undefined)
		this.indent = indent;
	this.bending = 0*PI; // Optional. How the angle of the wall is changed by this element for the following element, positive is bending 'in'
	if (bending !== undefined)
		this.bending = bending;
};

// Fortress object. A list would do for symetric fortresses but if 'getCenter' don't do sufficient the center can be set manually.
function fortress(type) // BAD NAME!!! To name a fortress type or a wallTool for a fortress "fortress" might be common!!!
{
	this.type = type; // Only usefull to get the type of the actual fortress (by 'wallTool.fortress.type')
	this.wall = [];
	this.center = [0, 0]; // X/Z offset (in default orientation) from first wall element to center, perhaps should be the other way around...
	// NOTE: The 'angle' is not implemented yet!!!
	this.angle = 0; // Angle the first element has to be placed so that fortress orientation is with main entrance top (default), should always be 0 though.
};

// The wall tool
function wallTool(style, fortressType)
{
	// General setup
	this.angle = 0; // Not really needed, may be useful to rotate fortresses after set...
	this.wallTypes = {}; // An associative array including all wall elements associated with 'wallElement.type'.
	this.wall = []; // A list of strings (wall element types) for simple wall building
	this.center = [0, 0] // X/Z offset from first wall element to center, perhaps should be the other way around...
	
	// Presets needed for wall style
	this.wallStyles = {};
	this.style = "palisades"; // Not really needed, perhaps helpfull to check current style
	if (style !== undefined)
		this.style = style;
	
	// Presets needed for tortess types
	this.fortressTypes = {};
	this.fortress = false; // Not really needed, perhaps usefull for getting fortress type, should be renamed somehow but simply 'type' is not very descriptive and 'fortressType' may be mistaken as 'fortressTypes'
	if (fortressType !== undefined)
		this.fortress = fortressType;
	this.calculateCenter = true; // If setFortress(Type) shall recalculate the center
	
	// Adding wall styles
	// Wall style 'palisades'
	this.wallStyles["palisades"] = {};
	// Straight unindented
	this.wallStyles["palisades"]["wall"] = new wallElement("wall", "other/palisades_rocks_straight", -PI/2, 2.5);
	this.wallStyles["palisades"]["tower"] = new wallElement("tower", "other/palisades_rocks_tower", -PI/2, 0.7);
	this.wallStyles["palisades"]["wallFort"] = new wallElement("wallFort", "other/palisades_rocks_fort", PI, 1.7);
	this.wallStyles["palisades"]["gate"] = new wallElement("gate", "other/palisades_rocks_gate", 0*PI, 3.6);
	this.wallStyles["palisades"]["endRight"] = new wallElement("endRight", "other/palisades_rocks_end", -PI/2, 0.2);
	this.wallStyles["palisades"]["endLeft"] = new wallElement("endLeft", "other/palisades_rocks_end", PI/2, 0.2);
	// Straight indented
	this.wallStyles["palisades"]["outpost"] = new wallElement("outpost", "other/palisades_rocks_outpost", PI, 0, -2);
	this.wallStyles["palisades"]["house"] = new wallElement("house", "other/celt_hut", PI, 0, 5);
	this.wallStyles["palisades"]["barracks"] = new wallElement("barracks", "other/celt_tavern", PI, 0, 5);
	this.wallStyles["palisades"]["entry"] = new wallElement("entry", "other/palisades_rocks_watchtower", 0*PI, 3.6, -2);
	this.wallStyles["palisades"]["wideEntry"] = new wallElement("wideEntry", "other/palisades_rocks_fort", PI, 6, 3);
	// Corners
	this.wallStyles["palisades"]["cornerIn"] = new wallElement("cornerIn", "other/palisades_rocks_curve", -PI/4, 2.1, 0.7, PI/2);
	this.wallStyles["palisades"]["cornerOut"] = new wallElement("cornerOut", "other/palisades_rocks_curve", PI/4, 2.1, -0.7, -PI/2);
	// Wall style 'cart'
	this.wallStyles["cart"] = {};
	this.wallStyles["cart"]["wall"] = new wallElement("wall", "structures/cart_wall", 0*PI, 6.2);
	this.wallStyles["cart"]["tower"] = new wallElement("tower", "structures/cart_wall_tower", PI, 2.7);
	this.wallStyles["cart"]["wallFort"] = new wallElement("wallFort", "structures/cart_fortress", PI, 5.1, 1.6);
	this.wallStyles["cart"]["outpost"] = new wallElement("outpost", "structures/cart_outpost", PI, 0, -5);
	this.wallStyles["cart"]["house"] = new wallElement("house", "structures/cart_house", PI, 0, 7);
	this.wallStyles["cart"]["gate"] = new wallElement("gate", "structures/cart_wall_gate", 0*PI, 6.2);
	this.wallStyles["cart"]["entry"] = new wallElement("entry", "structures/cart_defense_tower", PI, 6.2, -5);
	this.wallStyles["cart"]["wideEntry"] = new wallElement("wideEntry", "structures/cart_fortress", PI, 12, 7);
	this.wallStyles["cart"]["endRight"] = new wallElement("endRight", "structures/cart_wall_tower", PI, 2.7);
	this.wallStyles["cart"]["endLeft"] = new wallElement("endLeft", "structures/cart_wall_tower", PI, 2.7);
	this.wallStyles["cart"]["cornerIn"] = new wallElement("cornerIn", "structures/cart_wall_tower", 5*PI/4, 0.1, 0.9, PI/2);
	this.wallStyles["cart"]["cornerOut"] = new wallElement("cornerOut", "structures/cart_wall_tower", 3*PI/4, 1.3 /*1.6*/, 0, -PI/2);
	this.wallStyles["cart"]["barracks"] = new wallElement("barracks", "structures/cart_barracks", PI, 0, 7);
	// Wall style 'celt'
	this.wallStyles["celt"] = {};
	this.wallStyles["celt"]["wall"] = new wallElement("wall", "structures/celt_wall", 0*PI, 5, 0.5);
	this.wallStyles["celt"]["tower"] = new wallElement("tower", "structures/celt_wall_tower", PI, 0.7);
	this.wallStyles["celt"]["wallFort"] = new wallElement("wallFort", "structures/celt_fortress_g", PI, 4.2, 2);
	this.wallStyles["celt"]["outpost"] = new wallElement("outpost", "structures/celt_outpost", PI, 0, -5);
	this.wallStyles["celt"]["house"] = new wallElement("house", "structures/celt_house", PI, 0, 3);
	this.wallStyles["celt"]["gate"] = new wallElement("gate", "structures/celt_wall_gate", 0*PI, 3.2);
	this.wallStyles["celt"]["entry"] = new wallElement("entry", "structures/celt_defense_tower", PI, 6, -5);
	this.wallStyles["celt"]["wideEntry"] = new wallElement("wideEntry", "structures/celt_fortress_b", PI, 12, 7);
	this.wallStyles["celt"]["endRight"] = new wallElement("endRight", "structures/celt_wall_tower", PI, 0.7);
	this.wallStyles["celt"]["endLeft"] = new wallElement("endLeft", "structures/celt_wall_tower", PI, 0.7);
	this.wallStyles["celt"]["cornerIn"] = new wallElement("cornerIn", "structures/celt_wall_tower", PI, 0.7, 0, PI/2);
	this.wallStyles["celt"]["cornerOut"] = new wallElement("cornerOut", "structures/celt_wall_tower", PI, 0.7, 0, -PI/2);
	this.wallStyles["celt"]["barracks"] = new wallElement("barracks", "structures/celt_barracks", PI, 0, 7);
	// Wall style 'hele'
	this.wallStyles["hele"] = {};
	this.wallStyles["hele"]["wall"] = new wallElement("wall", "structures/hele_wall", 0*PI, 5.95);
	this.wallStyles["hele"]["tower"] = new wallElement("tower", "structures/hele_wall_tower", PI, 1.5);
	this.wallStyles["hele"]["wallFort"] = new wallElement("wallFort", "structures/hele_fortress", 2*PI/2 /* PI/2 */, 5.1 /* 5.6 */, 1.9 /* 1.9 */);
	this.wallStyles["hele"]["outpost"] = new wallElement("outpost", "structures/hele_outpost", PI, 0, -5);
	this.wallStyles["hele"]["house"] = new wallElement("house", "structures/hele_house", PI/2, 0, 6);
	this.wallStyles["hele"]["gate"] = new wallElement("gate", "structures/hele_wall_gate", 0*PI, 9.1);
	this.wallStyles["hele"]["entry"] = new wallElement("entry", "structures/hele_defense_tower", PI, 9.1, -4);
	this.wallStyles["hele"]["wideEntry"] = new wallElement("wideEntry", "structures/hele_fortress", 5*PI/2, 12, 7);
	this.wallStyles["hele"]["endRight"] = new wallElement("endRight", "structures/hele_wall_tower", PI, 1.5);
	this.wallStyles["hele"]["endLeft"] = new wallElement("endLeft", "structures/hele_wall_tower", PI, 1.5);
	this.wallStyles["hele"]["cornerIn"] = new wallElement("cornerIn", "structures/hele_wall_tower", 5*PI/4, 0, 0.5, PI/2);
	this.wallStyles["hele"]["cornerOut"] = new wallElement("cornerOut", "structures/hele_wall_tower", 3*PI/4, 1, 0, -PI/2);
	this.wallStyles["hele"]["barracks"] = new wallElement("barracks", "structures/hele_barracks", PI, 0, 6);
	// Wall style 'iber'
	this.wallStyles["iber"] = {};
	this.wallStyles["iber"]["wall"] = new wallElement("wall", "structures/iber_wall", 0*PI, 6.9);
	this.wallStyles["iber"]["tower"] = new wallElement("tower", "structures/iber_wall_tower", PI, 2.1);
	this.wallStyles["iber"]["wallFort"] = new wallElement("wallFort", "structures/iber_fortress", PI, 4.5, 0.7);
	this.wallStyles["iber"]["outpost"] = new wallElement("outpost", "structures/iber_outpost", PI, 0, -5);
	this.wallStyles["iber"]["house"] = new wallElement("house", "structures/iber_house", PI, 0, 4);
	this.wallStyles["iber"]["gate"] = new wallElement("gate", "structures/iber_wall_gate", 0*PI, 9.2);
	this.wallStyles["iber"]["entry"] = new wallElement("entry", "structures/iber_wall_tower", PI, 6.9, -5);
	this.wallStyles["iber"]["wideEntry"] = new wallElement("wideEntry", "structures/iber_fortress", PI/2, 9.2, 7);
	this.wallStyles["iber"]["endRight"] = new wallElement("endRight", "structures/iber_wall_tower", PI, 2.1);
	this.wallStyles["iber"]["endLeft"] = new wallElement("endLeft", "structures/iber_wall_tower", PI, 2.1);
	this.wallStyles["iber"]["cornerIn"] = new wallElement("cornerIn", "structures/iber_wall_tower", PI, 2.1, 0, PI/2);
	this.wallStyles["iber"]["cornerIn2"] = new wallElement("cornerIn2", "structures/iber_defense_tower", 5*PI/4, 1, 0.2, PI/2);
	this.wallStyles["iber"]["cornerIn3"] = new wallElement("cornerIn3", "structures/iber_wall_tower", 5*PI/4, 0.3, 0.5, PI/2);
	this.wallStyles["iber"]["cornerOut"] = new wallElement("cornerOut", "structures/iber_wall_tower", 3*PI/4, 1.3, 0, -PI/2);
	this.wallStyles["iber"]["barracks"] = new wallElement("barracks", "structures/iber_barracks", PI, 0, 7);
	// Wall style 'pers'
	this.wallStyles["pers"] = {};
	this.wallStyles["pers"]["wall"] = new wallElement("wall", "structures/pers_wall", 0*PI, 5.9);
	this.wallStyles["pers"]["tower"] = new wallElement("tower", "structures/pers_wall_tower", PI, 1.7);
	this.wallStyles["pers"]["wallFort"] = new wallElement("wallFort", "structures/pers_fortress", PI, 5.6/*5.5*/, 1.9/*1.7*/);
	this.wallStyles["pers"]["outpost"] = new wallElement("outpost", "structures/pers_outpost", PI, 0, -5);
	this.wallStyles["pers"]["house"] = new wallElement("house", "structures/pers_house", PI, 0, 6);
	this.wallStyles["pers"]["gate"] = new wallElement("gate", "structures/pers_wall_gate", 0*PI, 6);
	this.wallStyles["pers"]["entry"] = new wallElement("entry", "structures/pers_defense_tower", PI, 6, -4);
	this.wallStyles["pers"]["wideEntry"] = new wallElement("wideEntry", "structures/pers_fortress", PI, 12, 7);
	this.wallStyles["pers"]["endRight"] = new wallElement("endRight", "structures/pers_wall_tower", PI, 1.7);
	this.wallStyles["pers"]["endLeft"] = new wallElement("endLeft", "structures/pers_wall_tower", PI, 1.7);
	this.wallStyles["pers"]["cornerIn"] = new wallElement("cornerIn", "structures/pers_wall_tower", 5*PI/4, 0.2, 0.5, PI/2);
	this.wallStyles["pers"]["cornerOut"] = new wallElement("cornerOut", "structures/pers_wall_tower", 3*PI/4, 0.8, 0, -PI/2);
	this.wallStyles["pers"]["barracks"] = new wallElement("barracks", "structures/pers_barracks", PI, 0, 7);
	// Wall style 'rome'
	this.wallStyles["rome"] = {};
	this.wallStyles["rome"]["wall"] = new wallElement("wall", "structures/rome_wall", 0*PI, 5.9);
	this.wallStyles["rome"]["tower"] = new wallElement("tower", "structures/rome_wall_tower", PI, 2.1);
	this.wallStyles["rome"]["wallFort"] = new wallElement("wallFort", "structures/rome_fortress", PI, 6.3, 2.1);
	this.wallStyles["rome"]["outpost"] = new wallElement("outpost", "structures/rome_outpost", PI, 0, -5);
	this.wallStyles["rome"]["house"] = new wallElement("house", "structures/rome_house", PI, 0, 7);
	this.wallStyles["rome"]["gate"] = new wallElement("gate", "structures/rome_wall_gate", 0*PI, 5.9);
	this.wallStyles["rome"]["entry"] = new wallElement("entry", "structures/rome_defense_tower", PI, 5.9, -4);
	this.wallStyles["rome"]["wideEntry"] = new wallElement("wideEntry", "structures/rome_fortress", PI, 12, 7);
	this.wallStyles["rome"]["endRight"] = new wallElement("endRight", "structures/rome_wall_tower", PI, 2.1);
	this.wallStyles["rome"]["endLeft"] = new wallElement("endLeft", "structures/rome_wall_tower", PI, 2.1);
	this.wallStyles["rome"]["cornerIn"] = new wallElement("cornerIn", "structures/rome_wall_tower", 5*PI/4, 0, 0.7, PI/2);
	this.wallStyles["rome"]["cornerOut"] = new wallElement("cornerOut", "structures/rome_wall_tower", 3*PI/4, 1.1, 0, -PI/2);
	this.wallStyles["rome"]["barracks"] = new wallElement("barracks", "structures/rome_barracks", PI, 0, 6);
	
	// Setup some default fortress types
	this.fortressTypes["tiny"] = new fortress("tiny");
	var wallPart = ['entry', 'endLeft', 'wall', 'cornerIn', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["tiny"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["small"] = new fortress("small");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall',
		'cornerIn', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["small"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["medium"] = new fortress("medium");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'cornerIn', 'wall',
		'cornerOut', 'wall', 'cornerIn', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["medium"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["normal"] = new fortress("normal");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall',
		'cornerOut', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["normal"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["large"] = new fortress("large");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'tower', 'wall',
		'cornerOut', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["large"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["very large"] = new fortress("very large");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall',
		'tower', 'gate', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'wall',
		'cornerOut', 'wall', 'tower', 'wall', 'cornerIn', 'wall', 'tower', 'gate',
		'tower', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["very large"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["giant"] = new fortress("giant");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'outpost', 'wall', 'tower',
		'gate', 'tower', 'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'tower', 'wall',
		'cornerOut', 'wall', 'tower', 'wall', 'outpost', 'wall', 'cornerIn', 'wall', 'tower',
		'gate', 'tower', 'wall', 'outpost', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["giant"].wall.push(wallPart[wallIndex])
	};
	
	this.fortressTypes["demo"] = new fortress("demo");
	var wallPart = ['entry', 'endLeft', 'wall', 'tower', 'wall', 'wallFort', 'wall', 'tower', 'gate', 'tower', 'wall', 'outpost',
		'wall', 'cornerIn', 'wall', 'outpost', 'wall', 'endRight', 'wideEntry', 'endLeft', 'wall', 'tower', 'wall',
		'cornerOut', 'wall', 'tower', 'wall', 'endRight', 'wideEntry', 'endLeft', 'wall', 'outpost', 'wall', 'cornerIn',
		'wall', 'outpost', 'wall', 'tower', 'gate', 'tower', 'wall', 'wallFort', 'wall', 'tower', 'wall', 'endRight'];
	var parts = 4;
	for (var part = 0; part < parts; part++)
	{
		for (var wallIndex = 0; wallIndex < wallPart.length; wallIndex++)
			this.fortressTypes["demo"].wall.push(wallPart[wallIndex]);
	};
	
	// Setup style
	this.setStyle(this.style);
	
	// Setup fortress type if preasent
	if (this.fortress !== false && this.fortress !== undefined)
		this.setFortressType(this.fortress);
};

wallTool.prototype.setStyle = function(style)
{
	if (style in this.wallStyles)
	{
		this.style = style;
		this.wallTypes = this.wallStyles[style];
	}
	else
	{
		warn("Wall style '" + style + "' not found, falling back to 'palisades'...");
		this.style = 'palisades';
		this.wallTypes = this.wallStyles['palisades'];
	};
};

wallTool.prototype.getAlignment = function(startX, startZ, orientation)
{
	if (startX == undefined)
		startX = 0;
	if (startZ == undefined)
		startZ = 0;
	if (orientation == undefined)
		orientation = 0;
	var wallAlignment = [];
	var outside = orientation;
	var wallX = startX - this.center[1]*sin(orientation) - this.center[0]*cos(orientation);
	var wallZ = startZ - this.center[1]*cos(orientation) + this.center[0]*sin(orientation);
	for (var iWall = 0; iWall < this.wall.length; iWall++)
	{
		var element = this.wallTypes[this.wall[iWall]];
		var x = wallX - element.indent*sin(outside);
		var z = wallZ - element.indent*cos(outside);
		wallAlignment.push([x, z, element.entity, outside + element.angle]);
		// Presets for the next element
		if (iWall + 1 < this.wall.length)
		{
			outside += element.bending;
			var nextElement = this.wallTypes[this.wall[iWall + 1]];
			var distance = (element.width + nextElement.width)/2;
			// Indent corrections for corners
			if (element.bending !== 0 && element.indent !== 0)
			{
				// Indent correction of distance, not sure for non-right angles...
				distance += element.indent*sin(element.bending);
				// Indent correction of next normalized indentation
				wallX += element.indent * sin(outside);
				wallZ += element.indent * cos(outside);
			};
			wallX += distance * sin(outside + PI/2);
			wallZ += distance * cos(outside + PI/2);
		};
	};
	return wallAlignment
};

wallTool.prototype.getCenter = function(wallAlignment)
{
	var x = 0;
	var z = 0;
	for (var iWall = 0; iWall < wallAlignment.length; iWall++)
	{
		x += wallAlignment[iWall][0]/wallAlignment.length;
		z += wallAlignment[iWall][1]/wallAlignment.length;
	};
	var output = [x, z];
	return output;
};

wallTool.prototype.place = function(startX, startZ, playerId, orientation)
{
	var AM = this.getAlignment(startX, startZ, orientation);
	for (var iWall = 0; iWall < this.wall.length; iWall++)
	{
		if (AM[iWall][2] !== undefined)
			placeObject(AM[iWall][0], AM[iWall][1], AM[iWall][2], playerId, AM[iWall][3]);
	};
};

wallTool.prototype.setFortressType = function(type)
// type: A string of a default fortress type: "tiny", "small", "medium", "normal", "large", "very large", "giant"
{
	if (type in this.fortressTypes)
	{
		this.setFortress(this.fortressTypes[type]);
	}
	else
	{
		warn("Fortress type '" + type + "' not found, falling back to 'medium'");
		this.setFortress(this.fortressTypes["medium"]);
	};
};

wallTool.prototype.setFortress = function(fortress)
// NOTE: If you don't set "this.calculateCenter = false" the center is recalculated and reset.
// Fortress: An instance of a fortress class object
{
	this.fortress = fortress.type;
	this.wall = fortress.wall;
	if (this.calculateCenter == true)
	{
		this.center = [0, 0];
		var AM = this.getAlignment(0, 0, 0*PI);
		var center = this.getCenter(AM);
		this.center = center;
	}
	else
	{
		this.center = this.fortress.center
	};
};

wallTool.prototype.setGenericFortress = function(centerX, centerY, playerID, radius, maxBendOff, wall)
// WARNING: This function is far from optimal!
// radius: A float seting the aproximate radius of the fortress
// maxBendOff: Optional, The maximum random bending offset of the wall elements NOTE: If maxBendOff > PI/2 the wall might never close!!!
// wall: Optional, An array of wall element string, example: ["tower", "wall"] NOTE: Don't use wall elements with banding!!!
{
	if (wall == undefined)
		wall = ["tower", "wall"];
	if (maxBendOff > PI/2 || maxBendOff < 0)
		warn("setGenericFortress maxBendOff sould satisfy 0 < maxBendOff < PI/2 (~1.5) but it is: " + maxBendOff);
	if (maxBendOff == undefined)
		maxBendOff = 0;
	var wallLength = 0;
	for (var eleIndex = 0; eleIndex < wall.length; eleIndex++)
	{
		wallLength += this.wallTypes[wall[eleIndex]].width
	};
	if (radius < wallLength || wallLength < 0)
		warn("setGenericFortress: sum of the width of wall's elements should satisfy 0 < total length < 2*PI*radius but: radius = " + radius + ", wall = " + wall + " with total length of " + wallLength);
	
	var minEleWidth = 1000000; // Assuming the smallest element is about as wide as deep.
	for (var eleIndex = 0; eleIndex < wall.length; eleIndex++)
	{
		eleWidth = this.wallTypes[wall[eleIndex]].width;
		if (eleWidth < minEleWidth)
			minEleWidth = eleWidth;
	};
	var widthSafty = minEleWidth/4; // Can be done better...
	
	var x = wallLength/2;
	var y = radius;
	var angle = 0;
	var targetX = 0;
	var targetY = radius;
	var targetReached = false;
	var eleIndex = 0;
	this.wall = []; // not used
	while (targetReached == false && eleIndex < 4 * 2*PI * radius / wallLength)
	{
		var wallElement = this.wallTypes[wall[eleIndex % wall.length]];
		var eleWidth = wallElement.width - widthSafty;
		var eleAddAngleOff = randFloat(-maxBendOff, maxBendOff);
		var eleAddAngleWidth = cos(eleAddAngleOff) * eleWidth/2 / radius; // A/L = da/dl -> da = A * dl / L -> da = 2*PI * eleWidth / (2*PI * radius) -> da = eleWidth/radius
		var eleAngleAdd = eleAddAngleWidth + eleAddAngleOff;
		var eleAngle = angle + eleAngleAdd;
		var placeX = x + eleWidth/2 * cos(eleAngle);
		var placeY = y - eleWidth/2 * sin(eleAngle);
		var placeAngle = eleAngle + wallElement.angle;
		placeObject(centerX + placeX, centerY + placeY, wallElement.entity, playerID, placeAngle);
		x = placeX + eleWidth/2 * cos(eleAngle);
		y = placeY - eleWidth/2 * sin(eleAngle);
		angle += 2*eleAddAngleWidth;
		if (eleIndex % wall.length == 0 && eleIndex > wall.length && (getDistance(x, y, targetX, targetY) < wallLength || angle > 2*PI))
			targetReached = true;
		eleIndex++;
	};
};

wallTool.prototype.placeLinearWall = function(startX, startY, targetX, targetY, playerId, wallPart, style, endWithFirst)
// NOTE: Walls "front" or "outside" faces left when walking uppon the wall from "start" to "target".
// startX: x coordinate of the beginning of the wall
// startY: y coordinate of the beginning of the wall
// targetX: x coordinate of the ending of the wall
// targetY: y coordinate of the ending of the wall
// NOTE: Start and target coordinates are exact meaning the wall will begin/end at the given coordinates (not the first/last entity is placed there)
// playerId: Optional. The walls owners player ID (like in the function "placeObject" so 0 is gaia, 1 is the first player), Default is 0 (gaia)
// wallPart: Optional. An array of wall element type strings (see wallElement.type). Default is ["wall"]
// style: Optional. An wall style string (like defined in the "wallStyles" dictionary). Default is "palisades"
// endWithFirst: Optional. A boolean value. If true the 1st wall element in the wallPart array will finalize the wall. Default is true
{
	// Setup optional arguments
	if (playerId == undefined)
		playerId = 0;
	if (wallPart == undefined)
		wallPart = ["wall"];
	if (style == undefined)
		if (playerId == 0) // Meaning player is gaia
			style = "palisades"
		else
			style = g_MapSettings.PlayerData[playerId-1].Civ;
	if (endWithFirst == undefined)
		endWithFirst = true;
	// Setup number of wall parts
	var totalLength = getDistance(startX, startY, targetX, targetY);
	var wallPartLength = 0;
	for (var elementIndex = 0; elementIndex < wallPart.length; elementIndex++)
		wallPartLength += this.wallStyles[style][wallPart[elementIndex]].width;
	var numParts = 0;
	if (endWithFirst == true)
		numParts = ceil((totalLength - this.wallStyles[style][wallPart[0]].width) / wallPartLength)
	else
		ceil(numParts = totalLength / wallPartLength);
	// Setup scale factor
	var scaleFactor = 1;
	if (endWithFirst == true)
		scaleFactor = totalLength / (numParts * wallPartLength + this.wallStyles[style][wallPart[0]].width)
	else
		scaleFactor = totalLength / (numParts * wallPartLength);
	// Setup angle
	var wallAngle = getAngle(startX, startY, targetX, targetY); // NOTE: function "getAngle()" is about to be changed...
	var placeAngle = -wallAngle;
	// Place wall entities
	var x = startX;
	var y = startY;
	for (var partIndex = 0; partIndex < numParts; partIndex++)
	{
		for (var elementIndex = 0; elementIndex < wallPart.length; elementIndex++)
		{
			var wallEle = this.wallStyles[style][wallPart[elementIndex]];
			x += scaleFactor * wallEle.width/2 * cos(wallAngle);
			y += scaleFactor * wallEle.width/2 * sin(wallAngle);
			placeObject(x, y, wallEle.entity, playerId, placeAngle + wallEle.angle);
			x += scaleFactor * wallEle.width/2 * cos(wallAngle);
			y += scaleFactor * wallEle.width/2 * sin(wallAngle);
		};
	};
	if (endWithFirst == true)
	{
		var wallEle = this.wallStyles[style][wallPart[0]];
		x += wallEle.width/2 * cos(wallAngle);
		y += wallEle.width/2 * sin(wallAngle);
		placeObject(x, y, wallEle.entity, playerId, placeAngle + wallEle.angle);
	};
};

